from flask import Flask, request, abort, url_for
import telebot
import json
import time

from models import db, Chat

with open('config.json', 'r') as f:
    config = json.load(f)

app = Flask(__name__)
app.config.update(config)

db.app = app
db.init_app(app)

bot = telebot.TeleBot(config['bot']['token'], threaded=False)


def channel_commands(message, commands):
    bot_name = bot.get_me().username
    formatted_commands = ['/{cmd}@{usr}'.format(cmd=command, usr=bot_name) for command in commands]
    return message.text in formatted_commands

def oops(message, e):
    bot.send_message(message.chat.id, "Looks like something went wrong, send @skehmatics the following but note it may contain confidential information:")
    bot.send_message(message.chat.id, str(e))


@bot.channel_post_handler(func=lambda msg: channel_commands(msg, ['start']))
@bot.message_handler(commands=['start'])
def start(message):
    bot.reply_to(message, "Heyo, itsa me, the notify bot!")
    try:
        old_chat = Chat.query.filter_by(chat_id=message.chat.id).first()
        if old_chat:
            bot.send_message(message.chat.id, "It looks like you already have this bot activated with token `{token}`. If you want to revoke that token, use /revoke instead.".format(token=old_chat.token), parse_mode='Markdown')
            return
        chat = Chat(message.chat.id, message.chat.username)
        db.session.add(chat)
        db.session.commit()
        bot.send_message(chat.chat_id, "Here's your shiny new token: `{token}`".format(token=chat.token), parse_mode='Markdown')
        help(message, chat=chat)
    except Exception as e:
        oops(message, e)

@bot.channel_post_handler(func=lambda msg: channel_commands(msg, ['revoke']))
@bot.message_handler(commands=['revoke'])
def revoke(message, chat=None):
    if chat is None:
        chat = Chat.query.filter_by(chat_id=message.chat.id).first()
    try:
        chat_id = chat.chat_id
        db.session.delete(chat)
        db.session.commit()
        bot.send_message(chat_id, "Poof! Token for this chat is gone. Use /start to make a new one if you want")
    except Exception as e:
        oops(message, e)

@bot.channel_post_handler(func=lambda msg: channel_commands(msg, ['ping']))
@bot.message_handler(commands=['ping'])
def ping(message):
    bot.reply_to(message, "Pong!")

@bot.channel_post_handler(func=lambda msg: channel_commands(msg, ['help']))
@bot.message_handler(commands=['help'])
def help(message, chat=None):
    if chat is None:
        chat = Chat.query.filter_by(chat_id=message.chat.id).first()
    if chat is None:
        bot.send_message(message.chat.id, "You're not currently registered! Use /start to get all setup")
    else:
        bot.send_message(message.chat.id, """You can send messages to this chat via a post request like so\
\
```bash
curl -X POST -H "Content-Type: application/json" {url} --data '{{"text": "My cool message!"}}'
```\
If your token becomes compromised, or you don't want to use this bot anymore, use /revoke
If you're using this bot in a Channel, you'll have to append all commands with the bot username
See [the wiki](https://git.skehsucks.xyz/skeh/py_notify_bot/wiki) for more info""".format(url=url_for('send', token=chat.token, _external=True, _scheme='https')), parse_mode='Markdown')


@app.route('/')
def index():
    return "hello!\n"

@app.route('/send/<token>', methods=['POST'])
def send(token):
    if request.headers.get('content-type') == 'application/json':
        chat = Chat.query.filter_by(token=token).first()
        if chat:
            request_json = request.get_data().decode('utf-8')
            data = json.loads(request_json)
            bot.send_message(chat.chat_id, data.get('text'))
            return json.dumps({"status": "success"})
        else:
            return json.dumps({"status": "fail", "reason": "No such chat registered. Use the token provided by @py_notify_bot"})
    else:
        abort(415)


@app.route('/webhook/<key>/', methods=['POST'])
def webhook(key):
    if key == app.config['bot']['token']:
        if request.headers.get('content-type') == 'application/json':
            json_string = request.get_data().decode('utf-8')
            update = telebot.types.Update.de_json(json_string)
            bot.process_new_updates([update])
            return ''
        else:
            abort(415)
    else:
        abort(403)


if __name__ == '__main__':
    bot.remove_webhook()
    time.sleep(0.1)
    bot.set_webhook(url='https://{base}/webhook/{key}/'.format(base=app.config['bot']['base_url'], key=app.config['bot']['token']))

    app.run()
